"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toggleTa = exports.renewReader = exports.terminalUtils = exports.historyListener = exports.print = void 0;
const makeGetLastN_1 = require("../../shared/utils/makeGetLastN");
const printResult_1 = require("../../shared/utils/printResult");
const DO_AUTO_SCROLL = true;
const textAreas = [];
const latestTextArea = () => textAreas.length ? textAreas[textAreas.length - 1] : null;
const prompts = new Map();
const getLastN = (0, makeGetLastN_1.makeGetLastN)();
const callables = new Map();
const submitters = new Map();
const printAreas = new Map();
const whitelistedTypes = ['keydown'];
const allMaps = new Map;
const mapCallable = (textArea, fn) => {
    callables.set(textArea, fn);
};
const getCallable = (textArea) => {
    return callables.get(textArea);
};
const mapPrompt = (textArea, pr) => {
    prompts.set(textArea, pr);
};
const getPrompt = (textArea) => {
    return prompts.get(textArea);
};
const mapSubmitter = (textArea, fn) => {
    submitters.set(textArea, fn);
};
const getSubmitter = (textArea) => {
    return submitters.get(textArea);
};
const mapPrintArea = (textArea, elem) => {
    printAreas.set(textArea, elem);
};
const getPrintArea = (textArea) => {
    return printAreas.get(textArea);
};
const maps = {
    init: (name) => {
        allMaps.set(name, new Map());
    },
    sett: (mapName, key, value) => {
        const map1 = allMaps.get(mapName);
        if (!map1)
            return;
        map1.set(key, value);
    },
    get: (mapName, key) => {
        const map1 = allMaps.get(mapName);
        if (!map1)
            return;
        return map1.get(key);
    }
};
maps.init('toggled');
const taParent = (elem) => {
    return elem.parentElement;
};
const displayPrompt = (textArea) => {
    const prompt = getPrompt(textArea);
    const label = textArea.parentElement?.querySelector('.prompt-text');
    if (!label)
        throw new Error(`Can't find .prompt-text for the specificed <textarea>`);
    label.innerHTML = `<span>${prompt}</span>`;
};
const textAreaUtils = (textArea = latestTextArea()) => {
    return {
        get line() {
            return textArea.value;
        },
        matchUp: (obj) => {
            return obj.key === 'ArrowUp';
        },
        matchDown: (obj) => {
            return obj.key === 'ArrowDown';
        },
        eventName: 'keydown',
        clearCurrent: () => {
            textArea.value = '';
        },
    };
};
const readlineFunctions = (ta) => {
    const utils = textAreaUtils(ta);
    return {
        write: (arg) => ta.value = `${ta.value}${arg}`,
        close: () => { },
        get line() { return utils.line; },
        question: (pr, fn) => {
            mapPrompt(ta, pr);
            displayPrompt(ta);
            document.removeEventListener('keydown', handleKeypress, true);
            return new Promise((resolve) => {
                mapSubmitter(ta, async (arg) => {
                    document.removeEventListener('keydown', handleKeypress, true);
                    const res = await fn(arg);
                    return resolve(res);
                });
                document.addEventListener('keydown', handleKeypress, true);
            });
        }
    };
};
const getOrInitPrintArea = (ta) => {
    let printArea = getPrintArea(ta);
    if (!printArea) {
        const num = textAreas.indexOf(ta);
        if (num === -1)
            throw new Error('Error; could not find cached text area to match that one');
        const added = addElem('div', {
            class: `print-area print-area-${num}`,
            style: 'margin: 0; padding: 0;'
        });
        const text = addElem('pre', {
            class: `print-area-text print-area-text-${num}`,
        }, {
            parent: added
        });
        mapPrintArea(ta, text);
        printArea = getPrintArea(ta);
        if (ta?.parentElement?.parentElement === added.parentElement) {
            added.parentElement.appendChild(ta.parentElement);
            ta.focus();
        }
    }
    if (!printArea) {
        throw new Error('Could not find or create print area');
    }
    return printArea;
};
const print = (argv, arg, ta = latestTextArea()) => {
    const printArea = getOrInitPrintArea(ta);
    const text = (0, printResult_1.getText)(argv, arg);
    printArea.innerHTML = `${printArea.innerHTML}
${text}
`;
    if (DO_AUTO_SCROLL) {
        const printAreaContainer = getPrintArea(ta);
        const scrollable = printAreaContainer?.parentElement;
        if (scrollable) {
            scrollable.scroll({ top: printAreaContainer.getBoundingClientRect().height });
        }
    }
    return Promise.resolve(true);
};
exports.print = print;
const getLastTwo = (ta) => getLastN(ta, 2);
const addElem = (tag, attribs, opts = null) => {
    const options = opts ? opts : { parent: document.body };
    const elem = document.createElement(tag);
    Object.entries(attribs).forEach(([key, val]) => {
        elem.setAttribute(key, val);
    });
    const parentElem = options.parent;
    if (!parentElem || !parentElem.appendChild) {
        throw new Error(`Supposed parent element (${parentElem}) has no "appendChild" method`);
    }
    parentElem.appendChild(elem);
    return elem;
};
function isNyargsArea(elem) {
    return isTextArea(elem) && elem.classList.contains('ny-text-area');
}
function isTextArea(elem) {
    if (elem?.tagName === 'TEXTAREA')
        return true;
    return false;
}
const makeTextArea = () => {
    const taLen = textAreas.length;
    const parent = addElem('div', {
        style: 'overflow: visible;',
        class: `text-area-container text-area-container-${taLen}`,
        id: `text-area-container-${taLen}`
    });
    addElem('div', {
        'class': `prompt-text promp-text-${taLen}`,
        'style': 'position: absolute; right: 100%; width: 100%; top: 0; display: flex; justify-content: end;'
    }, {
        parent
    });
    const ta = addElem('textarea', {
        id: `textarea-${taLen}`,
        class: `prompt-text promp-text-${taLen} ny-text-area`,
        style: 'height: 100%; width: 100%;',
        autofocus: true
    }, {
        parent
    });
    if (!isTextArea(ta)) {
        throw new Error(`Cannot return non-textarea`);
    }
    maps.sett('toggled', ta, true);
    return ta;
};
exports.historyListener = {
    on: (evName, fn) => {
        if (whitelistedTypes.includes(evName) === false) {
            throw new Error('historyListener is only made for "keydown" kind of node readline spoofing.');
        }
        const latestTerminal = latestTextArea();
        mapCallable(latestTerminal, fn);
    }
};
exports.terminalUtils = {
    matchUp: (obj) => textAreaUtils().matchUp(obj),
    matchDown: (obj) => textAreaUtils().matchDown(obj),
    eventName: 'keydown',
    clearCurrent: () => { textAreaUtils().clearCurrent(); }
};
const renewReader = async (pr) => {
    let latestTerminal = latestTextArea();
    if (latestTerminal === null) {
        textAreas.push(makeTextArea());
        latestTerminal = latestTextArea();
    }
    if (latestTerminal === null)
        throw new Error('Could not find or create a textarea-based terminal');
    return readlineFunctions(latestTerminal);
};
exports.renewReader = renewReader;
async function handleKeypress(ke) {
    await handleTextKeypress(ke);
    handleAnyKeypress(ke)(true, async () => {
        const lastFour = getLastN(true, 4);
        if (lastFour.startsWith('-Control-Shift-;-')) {
            makeGetLastN_1.lastFive.map(ev => ev.preventDefault());
            const keypress = lastFour.split('-Control-Shift-;-')[1];
            const sel = `.ny-text-area#textarea-${keypress}`;
            const ta = document.querySelector(sel);
            if (ta && isNyargsArea(ta)) {
                const isOn = toggleTa(ta);
                if (isOn) {
                    ta.focus();
                }
                else {
                    ta.blur();
                }
            }
            else {
                throw new Error('naming clash; only text areas should have "ny-text-area" class');
            }
        }
    });
}
function handleAnyKeypress(ke) {
    return (elem, fn) => {
        if (elem === true || ke.target === elem) {
            fn();
        }
    };
}
function toggleTa(ta) {
    const toggled = maps.get('toggled', ta);
    const pa = taParent(ta);
    const printArea = getPrintArea(ta);
    const newTog = !toggled;
    if (newTog) {
        pa.classList.remove('is-offscreen');
        if (printArea?.parentElement)
            printArea.parentElement.classList.remove('is-offscreen');
    }
    else {
        pa.classList.add('is-offscreen');
        if (printArea?.parentElement)
            printArea.parentElement.classList.add('is-offscreen');
    }
    maps.sett('toggled', ta, newTog);
    return newTog;
}
exports.toggleTa = toggleTa;
async function handleTextKeypress(ke) {
    if (document.activeElement !== ke.target) {
        return;
    }
    if (!isNyargsArea(ke.target)) {
        return;
    }
    const histFunc = getCallable(ke.target);
    if (histFunc) {
        histFunc(null, ke);
    }
    const ta = ke.target;
    const last2 = getLastTwo(ta);
    if (last2 === '-Control-Enter') {
        const resolver = getSubmitter(ta);
        ke.stopPropagation();
        return resolver(ta.value);
    }
}
//# sourceMappingURL=index.js.map