"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setAll = void 0;
const files_1 = __importDefault(require("./files"));
const shared_1 = require("../shared");
const cliUtils_1 = require("./cliUtils");
const printResult_1 = require("../shared/utils/printResult");
const db_1 = require("./db");
const init_1 = __importDefault(require("./init"));
const defaultConfig = {
    browser: {
        hotkeys: {
            '-Control-Shift-K': () => (0, cliUtils_1.toggleTa)(document.querySelector('.ny-text-area'))
        },
        wrapperFn: (cmd) => { return cmd; },
        printResult: cliUtils_1.print,
        afterKeypress: async (ke) => { },
        processResult: async (result) => { return result; },
        messageUser: console.log,
        useFakeDb: false,
        init: init_1.default
    }
};
const setAll = async () => {
    Object.entries((defaultConfig.browser)).forEach(([key, val]) => {
        if ((0, shared_1.getConfig)(key) === undefined) {
            (0, shared_1.configure)('browser', key, val);
        }
    });
    shared_1.configured.resolver();
    (0, shared_1.set)('fs', {
        writeFile: files_1.default.write,
        readFile: files_1.default.read,
        mkdir: files_1.default.mkdir
    });
    const db = await (0, db_1.db)((0, shared_1.getConfig)('useFakeDb'));
    (0, shared_1.set)('db', db);
    (0, shared_1.set)('historyListener', cliUtils_1.historyListener);
    (0, shared_1.set)('renewReader', cliUtils_1.renewReader);
    (0, shared_1.set)('terminalUtils', cliUtils_1.terminalUtils);
    (0, shared_1.set)('printResult', printResult_1.printResult);
    (0, shared_1.set)('saveHistory', async (histState) => {
        await (await (0, db_1.filesDb)((0, shared_1.getConfig)('useFakeDb'))).files.put({
            name: 'history', data: { ...histState, idx: histState.hist.length }, updatedAt: Date.now()
        });
    });
    (0, shared_1.set)('loadHistory', async () => {
        const filesDb = await (0, db_1.filesDb)((0, shared_1.getConfig)('useFakeDb'));
        let file = await filesDb.files.get('history');
        if (file && file.data)
            return { ...file.data, line: '' };
        const sh = await (0, shared_1.get)('saveHistory');
        await sh({ hist: [], idx: 0 });
        file = await filesDb.files.get('history');
        if (!file.data)
            throw new Error(`Could not load history, even after trying to re-initialized fresh copy`);
        return { ...file.data, line: '' };
    });
};
exports.setAll = setAll;
(0, shared_1.set)('setAll', exports.setAll);
//# sourceMappingURL=deps.js.map