"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FAIL = exports.PASS = void 0;
const index_1 = require("../../../shared/index");
const dotprop_1 = __importDefault(require("dotprop"));
const string_argv_1 = __importDefault(require("string-argv"));
exports.PASS = 'PASS';
exports.FAIL = 'FAIL';
const arrEqual = (arr1, arr2) => {
    return arr1.length === arr2.length && arr1.filter((arrOneElem) => {
        return arr2.includes(arrOneElem);
    }).length === arr1.length;
};
const parseExpectation = (expected) => {
    if (expected === 'false')
        return false;
    if (expected === 'true')
        return true;
    if (expected === 'undefined')
        return undefined;
    if (expected === 'null')
        return null;
    return expected;
};
const assert = {
    help: {
        description: 'look up a value and assert a match based on "===". the assertion value is stored in the cache as  PASS or FAIL',
        options: {
            '[position one]': 'dot-property lookup within the commands and names namespaces',
            't:n': 'the cached names associated with the value to assert within; this is an array like --c:c',
            't:c': 'the cached commands associated with the value to assert within; this is an array like --c:n'
        },
        examples: {
            [`test assert 0.value.argv.l 1 --t:c match scalar --c:n testTests pass`]: 'find the array of cached object (returns from nyargs-based commands) from runs of ["match", "scalar"]; assert that the first (0th) has a property at .value.argv.l equals 1. if this is true, return "PASS", if not return "FAIL". --c:n means the names testTests and pass will be stored as the "names" caching namespace entries.'
        }
    },
    yargs: {
        't:c': {
            array: true,
        },
        't:n': {
            array: true,
        },
        where: {
            array: true,
        }
    },
    fn: async (args) => {
        const dexieQuery = {};
        const cmds = args['t:c'];
        const nms = args['t:n'];
        if (cmds && Array.isArray(cmds)) {
            dexieQuery.commands = cmds;
        }
        if (nms && Array.isArray(nms)) {
            dexieQuery.names = nms;
        }
        if (!dexieQuery.names && !dexieQuery.commands) {
            throw new Error(`commands or names is required for testing`);
        }
        const db = await (0, index_1.get)('db');
        const all = await db.cache.toArray();
        const arr = all.filter((elem) => {
            const nMatch = ((!dexieQuery.names)
                || elem.names === '*'
                || arrEqual(dexieQuery.names, elem.names));
            const cMatch = ((!dexieQuery.commands)
                || elem.commands === '*'
                || arrEqual(dexieQuery.commands, elem.commands));
            return nMatch && cMatch;
        });
        const checkPathRaw = args.positional[0];
        if (checkPathRaw.includes('[].')) {
            return whereResult(arr, args);
        }
        else {
            return normalResult(arr, args);
        }
    }
};
function normalResult(arr, args) {
    const testable = (0, dotprop_1.default)(arr, args.positional[0]);
    const expected = parseExpectation(args.positional[1]);
    const result = (testable !== expected) ? exports.FAIL : exports.PASS;
    const message = null;
    return {
        result: [result],
        commands: args.positional,
        message,
        expected,
        received: testable
    };
}
function whereResult(arr, args) {
    const where = args['where'];
    if (!where || where.length === 0)
        throw new Error('Error; a "where" argument must accompany use of "[]" in a test path');
    const [filterProp, filterVal] = (0, string_argv_1.default)(where[0]);
    if (filterVal === undefined || filterVal === 'undefined') {
        throw new Error(`Filtering (unlike asserting) for undefined on a dot-prop is disallowed (property of filter was ${filterProp}; val was undefined or "undefined")`);
    }
    const [prop, checkProp] = args.positional[0].split('[].');
    const subArray = (0, dotprop_1.default)(arr, prop);
    if (!Array.isArray(subArray)) {
        throw new Error(`Element at prop ${prop} is not an array; ${typeof subArray}`);
    }
    const testableParent = subArray.find((elem) => {
        return (0, dotprop_1.default)(elem, filterProp) === filterVal;
    });
    if (testableParent === undefined) {
        throw new Error(`Testing error; filtering a subarray on ${filterProp} did not turn up an object for the final drill-down.`);
    }
    const testable = (0, dotprop_1.default)(testableParent, checkProp);
    const expected = parseExpectation(args.positional[1]);
    const result = (testable !== expected) ? exports.FAIL : exports.PASS;
    let validAncestor = null;
    if (expected === undefined) {
        validAncestor = null;
        const validAncestorPropArr = checkProp.split('.');
        validAncestorPropArr.pop();
        if (validAncestorPropArr.length === 0)
            throw new Error(`The property following "[]" must be a compound`);
        const validAncestorProp = validAncestorPropArr.join('.');
        validAncestor = (0, dotprop_1.default)(testableParent, validAncestorProp);
        if (validAncestor === undefined) {
            throw new Error(`When checking for undefined, the final ancestor must not itself be undefined (in this case, ${validAncestorProp} from ${checkProp})`);
        }
    }
    return {
        result: [result],
        message: validAncestor ? `valid ancestor for undefined check ${validAncestor}` : null,
        commands: args.positional,
        expected,
        received: testable
    };
}
exports.default = assert;
//# sourceMappingURL=assert.js.map