"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDictionary = exports.setDictionary = exports.queue = exports.cliProgram = void 0;
const shared_1 = require("../../shared");
exports.cliProgram = {
    array: [],
    dictionary: {}
};
const isProgram = (arg) => {
    if (!Array.isArray(arg))
        return false;
    if (Array.length === 0)
        return false;
    if (arg.find(elem => typeof elem !== 'string')) {
        return false;
    }
    return true;
};
const isMultiplePrograms = (arg) => {
    if (!Array.isArray(arg))
        return false;
    return arg.find(arr => !isProgram(arr)) === undefined;
};
const queue = (stringOrProg) => {
    let lookedUpProg;
    if (typeof stringOrProg === 'string') {
        if (typeof exports.cliProgram.dictionary[stringOrProg] !== 'object') {
            throw new Error(`no program or invalid program at "${stringOrProg}"`);
        }
        lookedUpProg = exports.cliProgram.dictionary[stringOrProg];
    }
    else {
        lookedUpProg = stringOrProg;
    }
    exports.cliProgram.array = lookedUpProg;
};
exports.queue = queue;
const setDictionary = (newKeys) => {
    const msg = (args) => `Invalid program cannot be loaded: ${JSON.stringify(args)}`;
    Object.entries(newKeys).forEach(([nm, pr]) => {
        if (isMultiplePrograms(pr)) {
            pr.forEach((subpr, idx) => {
                const dict = { [`${nm}:${idx}`]: subpr };
                if (!isProgram(subpr))
                    throw new Error(msg(subpr));
                Object.assign(exports.cliProgram.dictionary, dict);
            });
        }
        else if (isProgram(pr)) {
            Object.assign(exports.cliProgram.dictionary, newKeys);
        }
        else {
            throw new Error(msg({ nm: pr }));
        }
    });
};
exports.setDictionary = setDictionary;
const getDictionary = () => {
    return exports.cliProgram.dictionary;
};
exports.getDictionary = getDictionary;
const m = {
    help: {
        description: 'run programs defined as arrays of cli commands for nyargs',
        examples: {
            'matchTwos': `Assuming "matchTwos" is defined as ['match scalar --l 2 --r 2'], run that one-line program with nyargs. To set the dictionary of programs, import loop from the index and in your program call, for example, "loop.setDictionary({ matchTwos: ['match scalar --l 2 --r 2'] })"`
        }
    },
    fn: async (args) => {
        const dict = (0, exports.getDictionary)();
        if (args.positional.length === 1) {
            if (args.positional[0] === 'List') {
                const log = (0, shared_1.getConfig)('messageUser');
                log(JSON.stringify(dict, null, 2));
                return null;
            }
        }
        const array = args.positional.map(progIdx => dict[progIdx]).flat();
        (0, exports.queue)(array);
        return array;
    }
};
exports.default = m;
//# sourceMappingURL=index.js.map