"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportDb = exports.importDb = exports.cache = exports.context = exports.dbPathUtil = void 0;
const store_1 = require("./store");
const dbPath_1 = require("../shared/utils/dbPath");
const idb_backup_and_restore_1 = require("../shared/idb-backup-and-restore");
const shared_1 = require("../shared");
exports.dbPathUtil = dbPath_1.dbPath;
exports.context = {
    currentPromise: null,
    currentResolve: null
};
const cacheResult = async (argv, data) => {
    if (!argv || data === undefined) {
        return;
    }
    const cacheDrilldown = argv.c
        ? argv.c
        : '.';
    if (typeof cacheDrilldown !== 'string') {
        return;
    }
    if ((argv._ || []).includes('cache')) {
        return;
    }
    const entry = {
        commands: argv['c:c']?.length
            ? argv['c:c'].map(cm => `${cm}`)
            : argv._.map(cm => `${cm}`),
        names: argv['c:n']?.length
            ? argv['c:n'].map(n => `${n}`)
            : [],
        value: data,
        filters: argv?.filters
    };
    await (0, store_1.put)(entry);
};
const cache = async (argv, data) => {
    if (argv.help) {
        console.log('escaping cache (help)');
        return;
    }
    if (!data.isMultiResult || !data.list) {
        return cacheResult(argv, data);
    }
    const proms = Object.entries(data.list)
        .map(([commandStr, cacheable]) => {
        const defaultArgv = { ...argv, 'c:c': commandStr.split(' ') };
        return cacheResult(defaultArgv, cacheable);
    });
    return Promise.all(proms);
};
exports.cache = cache;
async function importDb(path, f, dbBack) {
    const fs = await (0, shared_1.get)('fs');
    const idbDatabase = typeof f === 'string' ? dbBack : f;
    const builtPath = typeof f === 'string' ? (0, dbPath_1.dbPath)(path, f) : { fullpath: path };
    const { fullpath } = builtPath;
    const file = await fs.readFile(fullpath, 'utf8');
    console.log('importing db', idbDatabase);
    await (0, idb_backup_and_restore_1.clearDatabase)(idbDatabase);
    await (0, idb_backup_and_restore_1.importFromJson)(idbDatabase, file);
    console.log('imported from ', fullpath);
    return;
}
exports.importDb = importDb;
async function exportDb(p, f, dbBack) {
    const fs = await (0, shared_1.get)('fs');
    const { subdirs, fullpath } = (0, dbPath_1.dbPath)(p, f);
    await fs.mkdir(subdirs.join('/'), { recursive: true });
    const fname = fullpath;
    const str = await (0, idb_backup_and_restore_1.exportToJson)(dbBack);
    await fs.writeFile(fname, str, 'utf8');
    console.log(`wrote ${fname}; data: ${str}`);
    return fname;
}
exports.exportDb = exportDb;
//# sourceMappingURL=cache.js.map