"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInput = exports.fakeCli = void 0;
const index_1 = require("../shared/index");
const queue_1 = require("../shared/utils/queue");
const isNode_1 = __importDefault(require("../shared/utils/isNode"));
const makeGetLastN_1 = require("../shared/utils/makeGetLastN");
const setUp_1 = require("./setUp");
const const_1 = require("../shared/utils/const");
exports.fakeCli = {
    modules: null,
    handle: async (str) => {
        const fn = await setUp_1.caller.get;
        const answer = await fn(exports.fakeCli.modules, str);
        return await answer;
    }
};
const makeHandleQuestion = (res, modules) => {
    return function handleQuestion(rawPreInput) {
        let inp;
        const wrapperFn = (0, index_1.getConfig)('wrapperFn');
        const rawInput = wrapperFn(rawPreInput, modules);
        const splitted = rawInput.split('\n').filter(elem => !!elem);
        inp = splitted.shift();
        if (splitted.length) {
            (0, queue_1.queue)(splitted);
        }
        if (inp && inp.trim && inp.trim() && rawInput && rawInput !== 'undefined') {
            addHistory(rawInput);
        }
        curReadline.close();
        histIdx(histState.hist.length);
        return res(inp);
    };
};
const getLastN = (0, makeGetLastN_1.makeGetLastN)();
function recordKeypress(keyboardEvent) {
    makeGetLastN_1.lastFive.push(keyboardEvent);
    if (makeGetLastN_1.lastFive.length === 6) {
        makeGetLastN_1.lastFive.shift();
    }
}
const findKeypressMatch = async (hotkeys) => {
    const last2 = getLastN(true, 2);
    const last3 = getLastN(true, 3);
    const ret = Object.entries(hotkeys).find(([key, fn]) => [last2, last3].filter(elem => !!elem).includes(key));
    return ret;
};
let curReadline;
let didInitHistory = false;
let _getInput;
let histState = {
    hist: [],
    idx: 0,
    line: undefined
};
const addHistory = (line) => {
    if (line) {
        histState.hist.push(line);
    }
    (0, index_1.get)('saveHistory').then((saveHistory) => {
        saveHistory(histState);
    });
};
const loadHist = async () => {
    const lhFn = await (0, index_1.get)('loadHistory');
    const data = await lhFn();
    histState = data;
};
const histIdx = (idx) => {
    histState.idx = idx;
};
const initHistory = async (clearCurrent, write, historyListener, hs, utils) => {
    const { matchUp, matchDown, eventName } = utils;
    const hotkeys = (0, index_1.getConfig)('hotkeys');
    const afterKeypress = (0, index_1.getConfig)('afterKeypress');
    historyListener.on(eventName, async (_, obj) => {
        if (obj.type && obj.type !== eventName) {
            return;
        }
        recordKeypress(obj);
        const matches = await findKeypressMatch(hotkeys);
        if (matchUp(obj)) {
            if (hs.line || hs.idx === hs.hist.length) {
                addHistory(hs.line);
            }
            clearCurrent(curReadline);
            histIdx(Math.max(0, hs.idx - 1));
            if (hs.hist[hs.idx] === undefined)
                return;
            write(hs.hist[hs.idx]);
        }
        else if (matchDown(obj)) {
            histIdx(Math.min(hs.hist.length, hs.idx + 1));
            clearCurrent(curReadline);
            write(hs.hist[hs.idx] ?? '');
        }
        else if (matches && matches[1]) {
            matches[1](curReadline.line);
        }
        afterKeypress(obj);
    });
};
const getInput = async (modules, pr, initInput = '') => {
    const renewReader = await (0, index_1.get)('renewReader');
    const utils = await (0, index_1.get)('terminalUtils');
    const { clearCurrent } = utils;
    if (!_getInput) {
        _getInput = makeGetInput();
    }
    if (!curReadline) {
        curReadline = await renewReader(pr, curReadline);
    }
    if (!didInitHistory) {
        didInitHistory = true;
        const historyListener = await (0, index_1.get)('historyListener');
        await _getInput;
        initHistory(() => {
            clearCurrent(curReadline);
        }, (...args) => curReadline.write(...args), historyListener, histState, utils);
    }
    const fn = await _getInput;
    if (!(0, isNode_1.default)()) {
        clearCurrent(curReadline);
    }
    return fn(modules, pr, initInput);
};
exports.getInput = getInput;
const makeGetInput = async () => {
    if (_getInput)
        return _getInput;
    await loadHist();
    const renewReader = await (0, index_1.get)('renewReader');
    return async (modules, pr, initialInput = '') => {
        curReadline = await renewReader(pr, curReadline);
        const userInput = await new Promise((res) => {
            exports.fakeCli.modules = modules;
            curReadline.question(pr, makeHandleQuestion(res, modules));
            if (initialInput) {
                curReadline.write(initialInput);
                if (curReadline.line !== initialInput) {
                    curReadline.line = initialInput;
                }
            }
        });
        return userInput;
    };
};
//# sourceMappingURL=input.js.map