"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cache_1 = require("./cache");
const store_1 = require("./store");
const input_1 = require("./input");
const index_1 = require("../shared/index");
const queue_1 = require("../shared/utils/queue");
const index_2 = require("../shared/index");
const const_1 = require("../shared/utils/const");
const DO_LOG = true;
const log = (...args) => {
    if (DO_LOG) {
        console.log(...args);
    }
};
const PROMPT = 'nyargs > ';
let userPrompt = PROMPT;
const JQ_DEFAULT = 'l';
const containsInterrupt = (rawInput) => {
    if (rawInput && rawInput.includes && (rawInput.includes('davo:dismiss') || rawInput.includes('davo:dis'))) {
        return true;
    }
    return false;
};
const getExecuteCli = async (modules, yargsCaller) => async (input) => {
    let result;
    let argv;
    const x = await yargsCaller(modules, input || '');
    if (!x) {
        return;
    }
    result = x[const_1.RESULT_KEY];
    argv = x.argv;
    return { [const_1.RESULT_KEY]: result, argv };
};
async function verifyAndExecuteCli(modules, forwardedInput, executor, tempPrompt = userPrompt) {
    if (typeof const_1.RESULT_KEY !== 'string')
        throw new Error(`String required as RESULT_KEY`);
    const processResult = (0, index_2.getConfig)('processResult');
    let rawInput;
    let didUseProgram = false;
    const printResult = await (0, index_1.get)('printResult');
    if (queue_1.program.array.length) {
        rawInput = queue_1.program.array.shift();
        didUseProgram = true;
    }
    if (typeof rawInput === 'undefined') {
        rawInput = forwardedInput
            ? await (0, input_1.getInput)(modules, tempPrompt, forwardedInput)
            : await (0, input_1.getInput)(modules, tempPrompt);
    }
    if (containsInterrupt(rawInput)) {
        return { [const_1.RESULT_KEY]: {}, argv: {} };
    }
    if (!rawInput || !rawInput.split) {
        return verifyAndExecuteCli(modules, null, executor);
    }
    const input = (await (0, store_1.parseCacheInstructions)(rawInput, JQ_DEFAULT)).replace(/\{\{/g, '{{');
    if (!didUseProgram && input !== rawInput) {
        return verifyAndExecuteCli(modules, input, executor, 'AUGMENTED > ');
    }
    else {
        if (didUseProgram && input !== rawInput) {
            console.log('program line expanded: ', input);
        }
        const ret = await executor(input);
        await printResult(ret.argv, ret[const_1.RESULT_KEY]);
        await processResult(ret[const_1.RESULT_KEY]);
        await (0, cache_1.cache)(ret.argv, ret[const_1.RESULT_KEY]);
        return verifyAndExecuteCli(modules, null, executor);
    }
}
const repl = async (modules, yargsCaller, prompt) => {
    userPrompt = prompt ? prompt : userPrompt;
    const executeCli = await getExecuteCli(modules, yargsCaller);
    await verifyAndExecuteCli(modules, null, executeCli);
};
exports.default = repl;
//# sourceMappingURL=loop.js.map