"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.repl = exports.makeCaller = exports.caller = void 0;
const string_argv_1 = __importDefault(require("string-argv"));
const loop_1 = __importDefault(require("./loop"));
const help_1 = require("./help");
const index_1 = require("../shared/index");
const const_1 = require("../shared/utils/const");
const cliParser_1 = require("../shared/utils/cliParser");
const isNumber = (arg) => {
    return arg.match(/^(\-{0,1}[0-9]+\.[0-9]+|^\-{0,1}[0-9]+)$/) !== null;
};
const DO_LOG = false;
const log = (...args) => {
    if (DO_LOG) {
        console.log(...args);
    }
};
let lookUpAndCall_;
const yargsStarter = {
    'commands': {
        array: true,
        alias: 'c:c',
    },
    'names': {
        array: true,
        alias: 'c:n',
    }
};
const makeLookUpAndCall = async (yargs) => {
    lookUpAndCall_ = lookUpAndCall_ || lookUpCallFn;
    return lookUpAndCall_;
    async function lookUpCallFn(modules, input, commands) {
        let yargsOptions = {
            ...yargsStarter
        };
        let lastCommandFound = false;
        const lastPositionalOrNeg = input.findIndex(arg => {
            return arg.charAt(0) === '-'
                && !isNumber(arg);
        });
        const lastPositional = lastPositionalOrNeg > -1 ? lastPositionalOrNeg : input.length;
        let helpModule;
        let helpPrefix;
        let parentmostIsAsync;
        const namespaces = [];
        const positionals = [];
        let helpShown = false;
        let showHelp = () => (0, help_1.showModule)({ fn: async () => { }, submodules: modules });
        const reduced = commands.reduce((accum, curr, idx) => {
            if (accum.layer[curr] && !lastCommandFound) {
                const { submodules, help, fn, yargs: yOpts = {}, parallel } = accum.layer[curr];
                showHelp = () => (0, help_1.showModule)({ fn: fn, submodules }, `${accum.currentNamespace}`);
                if (parentmostIsAsync === undefined) {
                    parentmostIsAsync = !!parallel;
                    if (parentmostIsAsync) {
                        if (submodules === undefined || Object.values(submodules).length === 0) {
                            throw new Error(`Module properties error: A parallelizing module such as  ${curr} is required to have submodules (which it will pass to the parent ${curr}, which will control child-calling order from userland).`);
                        }
                    }
                }
                yargsOptions = { ...yargsOptions, ...yOpts };
                const newNs = `${accum.currentNamespace} ${curr}`.trim();
                namespaces.push(newNs);
                const wrapperFn = async (priors, isTop = false) => {
                    const opts1 = (0, cliParser_1.parse)(submodules, yargsOptions, input);
                    const cmdDepth = newNs.split(' ').length;
                    const positional = opts1._.slice(cmdDepth, lastPositional);
                    const underscore = opts1._.slice(0, cmdDepth);
                    const preferHelp = opts1.help === true;
                    const argv1 = {
                        ...opts1,
                        positional,
                        _: underscore
                    };
                    let result;
                    if (preferHelp) {
                        result = null;
                        helpModule = helpModule ?? accum.layer[curr];
                        helpPrefix = helpPrefix ?? newNs;
                    }
                    else {
                        if (parentmostIsAsync === false) {
                            await Promise.all(Object.values(priors));
                            result = await fn(argv1, priors);
                            log('result from fn()', result);
                        }
                        if (parentmostIsAsync === true) {
                            if (isTop) {
                                result = await fn(argv1, priors);
                                log('result from async parent fn()', result);
                            }
                            else {
                                const res = await fn(argv1, priors);
                                log('result from async child fn()', result);
                                return { [const_1.RESULT_KEY]: res };
                            }
                        }
                    }
                    return {
                        [const_1.RESULT_KEY]: result
                    };
                };
                return {
                    layer: submodules ?? accum.layer,
                    help,
                    fn: (fn) ? {
                        ...accum.fn,
                        [newNs]: wrapperFn,
                    } : accum.fn,
                    currentNamespace: newNs
                };
            }
            positionals.push(curr);
            lastCommandFound = true;
            return accum;
        }, {
            layer: modules,
            help: null,
            fn: {},
            currentNamespace: ''
        });
        const opts1 = (0, cliParser_1.parse)(modules, yargsOptions, input);
        const entries = Object.entries(reduced.fn);
        if (opts1.help === true) {
            showHelp();
        }
        if (entries.length === 0) {
            const ret = {
                isMultiResult: false,
                [const_1.RESULT_KEY]: {},
                errorInfo: null
            };
            if (namespaces.length === 0) {
                ret.errorInfo = 'No commands found to match input';
            }
            return ret;
        }
        return parentmostIsAsync ? await orderCallsAsync(entries, opts1) : await orderCallsSync(entries);
    }
};
async function orderCallsSync(entries) {
    entries.reverse();
    const mappedResults = {};
    const mappedArgv = {};
    let rollingProm = Promise.resolve();
    const proms = entries
        .map(async ([key, wrapperFn]) => {
        try {
            rollingProm = rollingProm.then(() => {
                return wrapperFn(mappedResults).then((r) => {
                    if (r !== undefined) {
                        mappedResults[key] = r[const_1.RESULT_KEY];
                        mappedArgv[key] = r.argv;
                    }
                });
            });
        }
        catch (cause) {
            throw new Error(`After ordering the call hierarchy and trying to trigger ${key}`, { cause });
        }
    });
    await Promise.all(proms);
    await rollingProm;
    return { isMultiResult: true, list: mappedResults, errorInfo: null };
}
async function orderCallsAsync(entries, opts1) {
    const topEntry = entries.shift();
    const result = await topEntry[1](entries, true);
    return {
        isMultiResult: false,
        [const_1.RESULT_KEY]: result,
        errorInfo: null
    };
}
let resolveCaller;
exports.caller = {
    get: new Promise((res) => {
        resolveCaller = res;
    })
};
const makeCaller = (yargs) => {
    const caller = async (modules, input) => {
        const lookUpAndCall = await makeLookUpAndCall(yargs);
        const simArgv = (0, string_argv_1.default)(input);
        const argv = (0, cliParser_1.parse)(modules, yargsStarter, simArgv);
        const commands = argv._;
        const result = await lookUpAndCall(modules, simArgv, commands);
        return { argv, [const_1.RESULT_KEY]: result };
    };
    resolveCaller(caller);
    return caller;
};
exports.makeCaller = makeCaller;
const repl = async (modules, yargs, prompt) => {
    const setAll = await (0, index_1.get)('setAll');
    await setAll();
    try {
        const caller = (0, exports.makeCaller)(yargs);
        return await (0, loop_1.default)(modules, caller, prompt);
    }
    catch (e) {
        const fullError = new Error('In loop.ts while running getExecuteCli:', { cause: e });
        console.error(fullError.message);
        console.error(fullError.stack);
        console.error(fullError.cause);
        lookUpAndCall_ = undefined;
        return await (0, exports.repl)(modules, yargs, prompt);
    }
};
exports.repl = repl;
//# sourceMappingURL=setUp.js.map