"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loggingExec = void 0;
const child_process_1 = require("child_process");
const util = require('node:util');
const exec = util.promisify(child_process_1.exec);
const log = console.log;
const awaitExit = async (runningProc, nextStepCheck) => {
    const codeMsg = (code) => 'child process exited with code ' + code.toString();
    const tryNextStep = (stdio) => {
        if (nextStepCheck !== null) {
            console.log('checking for', nextStepCheck, ' in ', stdio);
            if (stdio.includes(nextStepCheck)) {
                return true;
            }
        }
        return false;
    };
    const nextStepHandler = (data) => {
        const stdio = data.toString();
        const done = tryNextStep(stdio);
        if (done)
            return { msg: 'Going to next program step.' };
        return false;
    };
    runningProc.stderr.on('data', nextStepHandler);
    runningProc.stdout.on('data', nextStepHandler);
    runningProc.on('exit', (code) => {
        log('(on exit)', codeMsg(code));
        return { msg: codeMsg(code) };
    });
    runningProc.on('error', (code) => {
        log('(on error)');
        console.error(codeMsg(code));
        return { err: codeMsg(code) };
    });
};
const loggingExec = async ([cmd, args = []], { onLog = log, onError = console.error, nextStepCheck = null }) => {
    try {
        const spawned = (0, child_process_1.spawn)(cmd, args);
        spawned.stdout.on('data', function (data) {
            const stdout = data.toString();
            onLog(stdout);
        });
        spawned.stderr.on('data', function (data) {
            const stderr = data.toString();
            onError(stderr);
        });
        return awaitExit(spawned, nextStepCheck);
    }
    catch (e) {
        console.error(`could not run ${cmd}: ${e.message}`);
    }
};
exports.loggingExec = loggingExec;
//# sourceMappingURL=metaHelpers.js.map