"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setAll = void 0;
const node_fs_1 = require("node:fs");
const shared_1 = require("../shared");
const db_1 = require("./db");
const cliUtils_1 = require("./cliUtils");
const printResult_1 = require("../shared/utils/printResult");
require('source-map-support').install();
let attemptedHistory = false;
const defaultConfig = {
    server: {
        printResult: printResult_1.printResult,
        wrapperFn: (cmd) => {
            return cmd;
        },
        hotkeys: {
            '-Alt-Shift-T': (currInput) => {
            }
        },
        afterKeypress: async (ke) => { },
        processResult: async (result) => {
            return result;
        },
        messageUser: console.log
    }
};
const setAll = async () => {
    Object.entries((defaultConfig.server)).forEach(([key, val]) => {
        if (!(0, shared_1.getConfig)(key)) {
            (0, shared_1.configure)('server', key, val);
        }
    });
    shared_1.configured.resolver();
    (0, shared_1.set)('fs', {
        writeFile: node_fs_1.promises.writeFile,
        readFile: node_fs_1.promises.readFile,
        mkdir: node_fs_1.promises.mkdir
    });
    const db = await (0, db_1.db)();
    (0, shared_1.set)('db', db);
    (0, shared_1.set)('historyListener', {
        on: (_, fn) => {
            process.stdin.on('keypress', fn);
        }
    });
    (0, shared_1.set)('renewReader', cliUtils_1.renewReader);
    (0, shared_1.set)('terminalUtils', cliUtils_1.terminalUtils);
    (0, shared_1.set)('printResult', printResult_1.printResult);
    (0, shared_1.set)('saveHistory', async (histState) => {
        const fs = await (0, shared_1.get)('fs');
        const str = JSON.stringify({ ...histState, idx: histState.hist.length }, null, 2);
        fs.mkdir('data', { recursive: true });
        fs.writeFile('data/nyargs-history', str);
    });
    (0, shared_1.set)('loadHistory', async () => {
        const fs = await (0, shared_1.get)('fs');
        let hist;
        try {
            await fs.mkdir('data', { recursive: true });
            const strHist = await fs.readFile('data/nyargs-history', 'utf8');
            if (strHist) {
                return JSON.parse(strHist);
            }
            throw new Error('no good history found');
        }
        catch (e) {
            if (attemptedHistory === false) {
                attemptedHistory = true;
                await (0, shared_1.get)('saveHistory').then(async (shFn) => {
                    hist = { hist: [], idx: 0 };
                    await shFn(hist);
                    return hist;
                });
                const recursiveLhFn = await (0, shared_1.get)('loadHistory');
                return recursiveLhFn();
            }
            else {
                console.log('could not load or initialize persistent history!');
                return { hist: [], idx: 0 };
            }
        }
    });
};
exports.setAll = setAll;
(0, shared_1.set)('setAll', exports.setAll);
//# sourceMappingURL=deps.js.map