"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearDatabase = exports.importFromJson = exports.exportToJson = void 0;
function exportToJson(idbDatabase) {
    return new Promise((resolve, reject) => {
        const exportObject = {};
        if (idbDatabase.objectStoreNames.length === 0) {
            return resolve(JSON.stringify(exportObject, null, 2));
        }
        else {
            const list = idbDatabase.objectStoreNames;
            const transaction = idbDatabase.transaction(list, 'readonly');
            transaction.addEventListener('error', reject);
            for (const storeName of list) {
                const allObjects = [];
                transaction
                    .objectStore(storeName)
                    .openCursor()
                    .addEventListener('success', (event) => {
                    const cursor = event.target.result;
                    if (cursor) {
                        allObjects.push(cursor.value);
                        cursor.continue();
                    }
                    else {
                        exportObject[storeName] = allObjects;
                        if (idbDatabase.objectStoreNames.length ===
                            Object.keys(exportObject).length) {
                            resolve(JSON.stringify(exportObject, null, 2));
                        }
                    }
                });
            }
        }
    });
}
exports.exportToJson = exportToJson;
function importFromJson(idbDatabase, json) {
    return new Promise((resolve, reject) => {
        if (!idbDatabase)
            return resolve();
        const list = idbDatabase.objectStoreNames;
        const transaction = idbDatabase.transaction(list, 'readwrite');
        transaction.addEventListener('error', reject);
        var importObject = JSON.parse(json);
        for (const storeName of list) {
            let count = 0;
            if (importObject[storeName] === undefined) {
                continue;
            }
            if (importObject[storeName].length === 0) {
                delete importObject[storeName];
                continue;
            }
            for (const toAdd of importObject[storeName]) {
                const request = transaction.objectStore(storeName).put(toAdd);
                request.addEventListener('success', () => {
                    count++;
                    if (count === importObject[storeName].length) {
                        delete importObject[storeName];
                        if (Object.keys(importObject).length === 0) {
                            resolve();
                        }
                    }
                });
            }
        }
    });
}
exports.importFromJson = importFromJson;
function clearDatabase(idbDatabase) {
    return new Promise((resolve, reject) => {
        if (!idbDatabase)
            return resolve();
        if (typeof idbDatabase.objectStoreNames !== 'object') {
            throw new Error('an array is required');
        }
        const list = idbDatabase.objectStoreNames;
        const transaction = idbDatabase.transaction(list, 'readwrite');
        transaction.addEventListener('error', reject);
        let count = 0;
        for (const storeName of list) {
            transaction
                .objectStore(storeName)
                .clear()
                .addEventListener('success', () => {
                count++;
                if (count === idbDatabase.objectStoreNames.length) {
                    resolve();
                }
            });
        }
    });
}
exports.clearDatabase = clearDatabase;
//# sourceMappingURL=idb-backup-and-restore.js.map