"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.configure = exports.getConfig = exports.set = exports.get = exports.configured = exports.configuration = void 0;
const types_1 = require("./utils/types");
const isNode_1 = __importDefault(require("./utils/isNode"));
const depsRef = {} = {};
exports.configuration = {
    server: {},
    browser: {},
    shared: {}
};
exports.configured = {
    resolver: null,
    ready: null
};
exports.configured.ready = new Promise((resolve) => {
    exports.configured.resolver = () => { resolve(); };
});
const get = async (dn) => {
    if (dn !== 'setAll') {
        await exports.configured.ready;
    }
    if (isConfigOption(dn)) {
        const asConfigured = (0, exports.getConfig)(dn);
        if (asConfigured) {
            return asConfigured;
        }
    }
    if (!depsRef[dn])
        throw new Error(`No dep available at ${dn}`);
    return depsRef[dn];
};
exports.get = get;
const set = (depName, newDep) => {
    depsRef[depName] = newDep;
};
exports.set = set;
const isPlatform = (arg) => {
    return ['server', 'browser'].includes(arg);
};
const isConfigOption = (arg) => {
    return types_1.keyofConfigOptions.includes(arg);
};
const getConfig = (dn) => {
    const plat = (0, isNode_1.default)() ? 'server' : 'browser';
    if (Object.keys(exports.configuration[plat]).includes(dn)) {
        return exports.configuration[plat][dn];
    }
    else if (Object.keys(exports.configuration.shared).includes(dn)) {
        return exports.configuration.shared[dn];
    }
};
exports.getConfig = getConfig;
const configure = (optionOrPlatform, optionOrValue, value) => {
    let configSection;
    let settable;
    let settableVal;
    if (isPlatform(optionOrPlatform)) {
        if (isConfigOption(optionOrValue)) {
            settable = optionOrValue;
        }
        if (value === undefined)
            throw new Error('A third config argument is required when platform is supplied as the first.');
        settableVal = value;
        configSection = exports.configuration[optionOrPlatform];
    }
    else {
        configSection = exports.configuration.shared;
        settable = optionOrPlatform;
        settableVal = optionOrValue;
    }
    configSection[settable] = settableVal;
};
exports.configure = configure;
//# sourceMappingURL=index.js.map