"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.termGreenMsg = exports.grn = exports.cli = void 0;
const shelljs_1 = __importDefault(require("shelljs"));
const isPromise = (arg) => {
    return (typeof arg.then === 'function'
        && typeof arg.catch === 'function'
        && typeof arg.finally === 'function');
};
const stack = [];
const priorCall = (n) => {
    return stack.find(({ idx }, arrIdx) => {
        if (idx !== arrIdx) {
            throw new Error(`Library error; set idx should always equal the place in the stack`);
        }
        return idx === n - 1;
    });
};
const priorCalls = (n) => {
    return stack.filter(({ idx }, arrIdx) => {
        if (idx !== arrIdx) {
            throw new Error(`Library error; set idx should always equal the place in the stack`);
        }
        return idx < n;
    });
};
const allPriorPromises = (n) => {
    return priorCalls(n).map(({ promise }) => promise);
};
const cliCommandFromUnknown = async (cliCalled, commandOrString) => {
    if (typeof commandOrString === 'string')
        return commandOrString;
    let replaceFunctionResult;
    const prior = priorCall(cliCalled.idx);
    const directResult = commandOrString(`${prior.stdout}`);
    if (directResult === null) {
        shelljs_1.default.exec(`echo "exiting 1"`, () => {
            cliCalled.code = 1;
            cliCalled.stdout = prior.stdout;
            cliCalled.stderr = prior.stderr;
            cliCalled.isDone = true;
        });
        return '';
    }
    if (isPromise(directResult)) {
        replaceFunctionResult = await directResult;
    }
    else {
        replaceFunctionResult = directResult;
    }
    return replaceFunctionResult;
};
const functionInvariants = async (cliCalled, commands, ownIdx) => {
    const commandFn = commands[0];
    if (typeof commandFn !== 'function') {
        return [`Invariant failed; for "result(...)" call. a function must be passed as argument`, false];
    }
    const prior = priorCall(ownIdx);
    await prior.promise;
    if (prior.isDone !== true) {
        return [`In ".result(...) call, the prior was not finished! (prior call's data: ${prior.commands})`, false];
    }
    if (prior.code !== 0) {
        return [`Non-zero exit code on prior command: ${prior.stderr ?? prior.stdout}`, false];
    }
    if (prior.stderr) {
        return [`stderr was present on prior command: ${prior.stderr}`, false];
    }
    if (prior.stdout && prior.stderr)
        return [`Unknown case in which stdout and stderr both had truthy values; stdout: ${prior.stdout} ; stderr: ${prior.stderr}`, false];
    if (prior.stderr || prior.code !== 0) {
        const msg = "failing; prior command(s) call ${prior.commands} has truthy stderr or nonzero code. (Code was ${prior.code}.)";
        shelljs_1.default.exec(`echo "${msg}"`, () => {
            cliCalled.code = prior.code;
            cliCalled.stdout = prior.stdout;
            cliCalled.stderr = prior.stderr;
            cliCalled.isDone = true;
        });
        return [msg, false];
    }
    const replaceFunctionResult = await cliCommandFromUnknown(cliCalled, commands[0]);
    if (typeof replaceFunctionResult !== 'string') {
        return [`A string was expected.`, false];
    }
    return [replaceFunctionResult, true];
};
function executeCommand(commands, callType) {
    const ownIdx = stack.length;
    const cliCalled = {
        idx: ownIdx,
        promise: null,
        isDone: false,
        code: null,
        stdout: null,
        stderr: null,
        isFirst: false,
        commands,
        cli: (nextCmd) => {
            return executeCommand([nextCmd], 'cli');
        },
        replace: (fn) => {
            return executeCommand([fn], 'replace');
        },
        wait: (fnOrString) => {
            return executeCommand([fnOrString], 'wait');
        }
    };
    if (callType === 'replace') {
        cliCalled.promise = new Promise(async (res, rej) => {
            const [replaceFunctionResult, doRun] = await functionInvariants(cliCalled, commands, ownIdx);
            if (doRun) {
                cliCalled.commands.pop();
                cliCalled.commands.push(replaceFunctionResult);
                shelljs_1.default.exec(replaceFunctionResult, (code, stdout, stderr) => {
                    cliCalled.code = code;
                    cliCalled.stdout = stdout ?? null;
                    cliCalled.stderr = stderr ?? null;
                    cliCalled.isDone = true;
                    return res();
                });
            }
            else {
                return rej(replaceFunctionResult);
            }
        });
    }
    else if (callType === 'wait') {
        cliCalled.promise = priorCall(cliCalled.idx).promise.then(async () => {
            const cmd = await cliCommandFromUnknown(cliCalled, commands[0]);
            return new Promise(async (res) => {
                const prior = priorCall(ownIdx);
                await prior.promise;
                shelljs_1.default.exec(cmd, (code, stdout, stderr) => {
                    cliCalled.code = code;
                    cliCalled.stdout = stdout ?? null;
                    cliCalled.stderr = stderr ?? null;
                    cliCalled.isDone = true;
                    return res();
                });
            });
        });
    }
    else if (callType === 'cli') {
        cliCalled.promise = new Promise(async (res, rej) => {
            const allProms = allPriorPromises(ownIdx);
            await Promise.all(allProms);
            const priorErrors = priorCalls(ownIdx).map(({ stderr, code, stdout, commands }) => {
                return code !== 0 ? (stderr ?? stdout ?? `Exiting due to failure in a prior result. The exact cause is unknown, but here is some info: ${commands.toString()} `) : null;
            }).filter((result) => {
                return result !== null;
            });
            if (priorErrors.length) {
                return shelljs_1.default.exec('echo "exiting 2"', (code, stdout) => {
                    cliCalled.code = code;
                    cliCalled.stdout = stdout ?? null;
                    cliCalled.stderr = `Aborted before ${commands[0]} ; priorErrors.join("; ")}`;
                    cliCalled.isDone = true;
                    return res();
                });
            }
            else {
                try {
                    if (typeof commands[0] === 'string') {
                        return shelljs_1.default.exec(commands[0], (code, stdout, stderr) => {
                            cliCalled.code = code;
                            cliCalled.stdout = stdout ?? null;
                            cliCalled.stderr = stderr ?? null;
                            cliCalled.isDone = true;
                            res();
                            return;
                        });
                    }
                    else
                        throw new Error(`Non-string command to cli`);
                }
                catch (e) {
                    return rej(e);
                }
            }
        });
    }
    stack.push(cliCalled);
    return cliCalled;
}
function cli(command) {
    return executeCommand([command], 'cli');
}
exports.cli = cli;
const grn = (str) => `\x1b[32m${str}\x1b[0m`;
exports.grn = grn;
const termGreenMsg = (str) => console.log(`\x1b[32m${str}\x1b[0m`);
exports.termGreenMsg = termGreenMsg;
//# sourceMappingURL=cli.js.map