"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.destructureSwipl = exports.destructureAnswers = void 0;
const localLogger = console.log;
const isTermArg = (arg) => {
    if (arg.index === undefined || arg.ground === undefined)
        return true;
    const isTerm = (typeof arg.value === 'undefined'
        && (arg.id !== '.' && (["[]/0", "./2"].includes(arg.indicator) === false)));
    return isTerm;
};
const isNonListArg = (arg) => {
    if (arg.index === undefined || arg.ground === undefined)
        return true;
    const isNonList = (typeof arg.value !== 'undefined'
        || isTermArg(arg));
    return isNonList;
};
const isListArg = (arg) => {
    return typeof arg.is_float === 'undefined';
};
const reduceToArray = (obj, contexts, predicates) => {
    localLogger('called reduce to array', obj);
    if (!isListArg(obj))
        throw new Error('Not a listy arg');
    if (obj.args.length === 0) {
        localLogger('0 length arg passed');
        return;
    }
    if (obj.args.length > 2) {
        throw new Error('Unhandled case when traversing a prolog list (0)');
    }
    const [first, second] = obj.args;
    const isNonList = isNonListArg(first);
    const tail = isListArg(second) && second.indicator || null;
    localLogger('entering switch; contexts:', JSON.stringify(contexts));
    switch (isNonList) {
        case true:
            localLogger('pushing in a non-list value from object', first);
            contexts[contexts.length - 1].push(first.index);
            if (isTermArg(first)) {
                predicates.push(first.indicator);
            }
            switch (tail) {
                case "[]/0":
                    localLogger('popping a context (head was non-list value-ish');
                    return;
                case "./2":
                    localLogger('next element in tree...(a)');
                    reduceToArray(second, contexts, predicates);
                    localLogger('back from (a) ; conctexts and results', JSON.stringify({ contexts }));
                    return;
                default:
                    throw 'Unhandled case when traversing prolog list (1)';
            }
        case false:
            localLogger('found a list element as first in tree; pushing new context');
            contexts.push([]);
            localLogger('with new context, handling list.');
            reduceToArray(first, contexts, predicates);
            const parent = contexts[contexts.length - 2];
            parent.push(contexts.pop());
            switch (tail) {
                case "[]/0":
                    localLogger('empty tail list found; popping context. head was listy');
                    return;
                case "./2":
                    localLogger('tail list with elements found; going deeper.');
                    reduceToArray(second, contexts, predicates);
                    return;
                default:
                    throw 'Unhandled case when traversing prolog list (2)';
            }
    }
};
const destructureAnswer = (ans, contexts) => {
    localLogger('called destructureAnswer', ans);
    const links = ans.links;
    const predicates = [];
    const destructured = Object.entries(links).reduce((accum, [key, obj]) => {
        if (isNonListArg(obj))
            return { ...accum, [key]: obj };
        contexts.push([]);
        reduceToArray(obj, contexts, predicates);
        return { ...accum, [key]: contexts.pop() };
    }, {});
    return { predicates, destructured };
};
const destructureAnswers = (ans) => {
    localLogger('destructureAnswers', JSON.stringify(ans, null, 2));
    const contexts = [];
    const tauLinks = ans.map(anAns => anAns.json).filter((resp) => resp !== 'fail');
    const destructured = tauLinks.map((ans) => destructureAnswer(ans, contexts)).filter((destruct) => destruct !== null);
    return destructured;
};
exports.destructureAnswers = destructureAnswers;
const isSwiplBindings = (arg) => {
    if (Object.values(arg).find((anArg) => {
        return !isSwiplVal(anArg);
    })) {
        return false;
    }
    return true;
};
const isSwiplScalar = (arg) => {
    if (arg === '[]')
        return false;
    return typeof arg === 'string' || typeof arg === 'number';
};
const isSwiplHT = (arg) => {
    if (arg === null || typeof arg !== 'object')
        return false;
    if (arg.head === undefined)
        return false;
    if (arg.tail === undefined)
        return false;
    if (arg.head === null)
        return true;
    return isSwiplVal(arg.head) && isSwiplVal(arg.tail);
};
const isSwiplCompound = (arg) => {
    if (arg === null) {
        return false;
    }
    if (!arg.name || !arg.args)
        return false;
    if (!isSwiplScalar(arg.name))
        return false;
    if (!Array.isArray(arg.args))
        return false;
    return arg.args.find((anArg) => {
        return !isSwiplVal(anArg);
    }) === undefined;
};
const isSwiplVal = (arg) => {
    const truth = isSwiplEmptyList(arg) || isSwiplScalar(arg) || isSwiplHT(arg) || isSwiplCompound(arg);
    if (truth)
        return true;
    if (!truth) {
        console.log('not a swipl val', arg);
    }
    return truth;
};
const isSwiplEmptyList = (arg) => {
    return arg === '[]';
};
const destructureS = (arg) => {
    if (isSwiplEmptyList(arg)) {
        return [];
    }
    else if (isSwiplScalar(arg)) {
        return arg;
    }
    else if (isSwiplHT(arg)) {
        const { head, tail } = arg;
        const flattenedHead = head === null ? null : destructureS(head);
        const flattenedTail = destructureS(tail);
        if (Array.isArray(flattenedTail)) {
            return [flattenedHead, ...flattenedTail];
        }
        return [flattenedHead, flattenedTail];
    }
    else if (isSwiplCompound(arg)) {
        const { name, args: cmpdArgs } = arg;
        const flattenedArgs = cmpdArgs.map(destructureS);
        return { name: name.toString(), args: flattenedArgs };
    }
    throw new Error(`Could not detect this to be a swipl value: ${arg}`);
};
const destructureSwipl = (data) => {
    if (!data)
        throw new Error('Cannot destructure null or undefined');
    data.forEach((elem) => {
        if (!isSwiplBindings(elem))
            throw new Error(`Not a set of swipl bindings: ${JSON.stringify(data)}`);
    });
    return data.reduce((accum, curr) => {
        const entries = Object.entries(curr);
        return [
            ...accum, {
                ...entries.reduce((accum, [varName, swiplVal]) => {
                    return {
                        ...accum,
                        [varName]: destructureS(swiplVal)
                    };
                }, {})
            }
        ];
    }, []);
};
exports.destructureSwipl = destructureSwipl;
//# sourceMappingURL=swiplAdaptors.js.map