"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.destructureAnswers = void 0;
const localLogger = console.log;
const isTermArg = (arg) => {
    if (arg.index === undefined || arg.ground === undefined)
        return true;
    const isTerm = (typeof arg.value === 'undefined'
        && (arg.id !== '.' && (["[]/0", "./2"].includes(arg.indicator) === false)));
    return isTerm;
};
const isNonListArg = (arg) => {
    if (arg.index === undefined || arg.ground === undefined)
        return true;
    const isNonList = (typeof arg.value !== 'undefined'
        || isTermArg(arg));
    return isNonList;
};
const isListArg = (arg) => {
    return typeof arg.is_float === 'undefined';
};
const reduceToArray = (obj, contexts, predicates) => {
    localLogger('called reduce to array', obj);
    if (!isListArg(obj))
        throw new Error('Not a listy arg');
    if (obj.args.length === 0) {
        localLogger('0 length arg passed');
        return;
    }
    if (obj.args.length > 2) {
        throw new Error('Unhandled case when traversing a prolog list (0)');
    }
    const [first, second] = obj.args;
    const isNonList = isNonListArg(first);
    const tail = isListArg(second) && second.indicator || null;
    localLogger('entering switch; contexts:', JSON.stringify(contexts));
    switch (isNonList) {
        case true:
            localLogger('pushing in a non-list value from object', first);
            contexts[contexts.length - 1].push(first.index);
            if (isTermArg(first)) {
                predicates.push(first.indicator);
            }
            switch (tail) {
                case "[]/0":
                    localLogger('popping a context (head was non-list value-ish');
                    return;
                case "./2":
                    localLogger('next element in tree...(a)');
                    reduceToArray(second, contexts, predicates);
                    localLogger('back from (a) ; conctexts and results', JSON.stringify({ contexts }));
                    return;
                default:
                    throw 'Unhandled case when traversing prolog list (1)';
            }
        case false:
            localLogger('found a list element as first in tree; pushing new context');
            contexts.push([]);
            localLogger('with new context, handling list.');
            reduceToArray(first, contexts, predicates);
            const parent = contexts[contexts.length - 2];
            parent.push(contexts.pop());
            switch (tail) {
                case "[]/0":
                    localLogger('empty tail list found; popping context. head was listy');
                    return;
                case "./2":
                    localLogger('tail list with elements found; going deeper.');
                    reduceToArray(second, contexts, predicates);
                    return;
                default:
                    throw 'Unhandled case when traversing prolog list (2)';
            }
    }
};
const destructureAnswer = (ans, contexts) => {
    localLogger('called destructureAnswer', ans);
    const links = ans.links;
    const predicates = [];
    const destructured = Object.entries(links).reduce((accum, [key, obj]) => {
        if (isNonListArg(obj))
            return { ...accum, [key]: obj };
        contexts.push([]);
        reduceToArray(obj, contexts, predicates);
        return { ...accum, [key]: contexts.pop() };
    }, {});
    return { predicates, destructured };
};
const destructureAnswers = (ans) => {
    localLogger('destructureAnswers', JSON.stringify(ans, null, 2));
    const contexts = [];
    const tauLinks = ans.map(anAns => anAns.json).filter((resp) => resp !== 'fail');
    const destructured = tauLinks.map((ans) => destructureAnswer(ans, contexts)).filter((destruct) => destruct !== null);
    return destructured;
};
exports.destructureAnswers = destructureAnswers;
//# sourceMappingURL=tauAdaptors.js.map