import { RESULT_KEY } from "./const";
export declare type SaveHistory = (histState: {
    hist: string[];
    idx: number;
}) => Promise<void>;
export declare type LoadHistory = () => Promise<{
    hist: string[];
    idx: number;
}>;
export interface Files {
    read: (path: string, opts?: unknown) => Promise<string>;
    write: (arg1: string, arg2: string, arg3?: unknown) => Promise<void>;
    mkdir: (arg1: string, arg2?: string | {
        recursive?: boolean;
    }) => Promise<string>;
}
export declare type PartialFs = {
    readFile: Files['read'];
    writeFile: Files['write'];
    mkdir: Files['mkdir'];
};
export interface Cache {
    id?: number;
    names: Array<string> | '*';
    commands: Array<string> | '*';
    value: any;
    createdAt: number;
}
export interface DbFiles {
    name: string;
    data: any;
    updatedAt: number;
}
export interface Hotkeys {
    [keyCombo: string]: (currInput: any) => void;
}
export declare type Db = any;
export interface TerminalUtils {
    default?: TerminalUtils;
    matchUp: (arg1: any) => boolean;
    matchDown: (arg1: any) => boolean;
    eventName: string;
    clearCurrent: (rl?: ReadlineInterface) => void;
}
export interface ReadlineInterface {
    question: Function;
    write: Function;
    close: Function;
    line: string;
}
export interface SingleResult {
    isMultiResult: false;
    errorInfo: null | string;
    [RESULT_KEY]: any;
}
interface MultiResult {
    isMultiResult: true;
    errorInfo: null | string;
    list: {
        [str: string]: SingleResult;
    };
}
export declare type Result = SingleResult | MultiResult;
export interface PrintResult<T extends BaseArguments = BaseArguments> {
    (argv: T, res: Result): Promise<boolean>;
}
export declare type Readline = {
    default?: Readline;
    createInterface: (arg: {
        input: any;
        output: any;
        prompt: any;
    }) => ReadlineInterface;
    utils?: {
        matchUp: Function;
        matchDown: Function;
        eventName: string;
    };
    getInput?: (arg1: string, arg2?: string) => Promise<string>;
};
export interface HistoryListener {
    on: Function;
}
export interface RenewReader {
    (arg1: string, arg2: ReadlineInterface, htmlContainer?: any): Promise<ReadlineInterface>;
}
export interface ModuleHelp {
    description: string;
    options?: object;
    examples?: object;
}
export declare type JsonObjects = Array<string>;
declare type SubmoduleResult = ReturnType<Module['fn']>;
export declare type Modules = Record<string, Module>;
declare type AnyModule = {
    fn: ModuleFn<any, any>;
    help?: ModuleHelp;
    yargs?: {
        [optName: string]: any;
    };
    submodules?: {
        [submoduleName: string]: AnyModule;
    };
};
export declare type SyncChildCalls = {
    [childCommandNames: string]: SubmoduleResult;
};
export declare type ParallelChildCalls = [childCommandNames: string, submoduleWrapperFn: Function][];
export declare type ModuleFn<X, ReturnType> = (argv: BaseArguments & X, submodulePromises: SyncChildCalls) => Promise<ReturnType>;
export declare type ParallelModuleFn<X, ReturnType = any> = (argv: BaseArguments & X, submodulePromises: ParallelChildCalls) => Promise<ReturnType>;
export declare type Module<T = any, R = any> = SyncModule<T, R>;
export declare type SyncModule<UserArguments = {}, ReturnType = any> = {
    fn: ModuleFn<UserArguments, ReturnType>;
    help?: ModuleHelp;
    yargs?: {
        [optName: string]: any;
    };
    parallel?: false;
    submodules?: {
        [submoduleName: string]: AnyModule;
    };
};
export declare type ParallelModule<UserArguments = {}, ReturnType = any> = {
    fn: ParallelModuleFn<UserArguments, ReturnType>;
    help?: ModuleHelp;
    yargs?: {
        [optName: string]: any;
    };
    parallel: true;
    submodules: {
        [submoduleName: string]: AnyModule;
    };
};
export declare type ModuleArgs<T = {}> = Parameters<Module<T>['fn']>;
export interface BaseArguments {
    'c:c': string[];
    'c:n': string[];
    filters: string[];
    positional: (string | number)[];
    _: (string | number)[];
    c?: any;
    help?: boolean;
    logArgs?: boolean;
}
export interface NodeKeypress {
    meta: boolean;
    ctrl: boolean;
    shift: boolean;
    sequence: string;
    name: string;
}
export interface ConfigOptions {
    printResult?: PrintResult;
    wrapperFn?: (cmd: string, modules?: Modules) => string;
    hotkeys?: Hotkeys;
    afterKeypress?: (ke: KeyboardEvent | NodeKeypress) => Promise<void>;
    processResult?: (result: Result) => Promise<any>;
    messageUser?: (...messages: any) => void;
    useFakeDb?: boolean;
    init?: (...args: any[]) => any;
}
export interface Configuration {
    shared?: ConfigOptions;
    browser?: ConfigOptions;
    server?: ConfigOptions;
}
export declare const keyofConfigOptions: (keyof ConfigOptions)[];
export declare type SetAll = () => Promise<void>;
export declare type Repl = (modules: {
    [moduleName: string]: Module<any, any> | ParallelModule<{}, any>;
}, yargs: any, prompt: string) => Promise<any>;
export {};
